from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class Pdf3DAnnotation(PdfFileAnnotation):
    """
    Represents the 3D annotation for a PDF document.
    """
    @dispatch
    def __init__(self, rectangle: RectangleF):
        """
        Initializes a new instance of the Pdf3DAnnotation class with the specified rectangle.
        
        Args:
            rectangle (RectangleF): The rectangle of the annotation.
        """
        ptrRec: c_void_p = rectangle.Ptr

        GetDllLibPdf().Pdf3DAnnotation_CreatePdf3DAnnotationR.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DAnnotation_CreatePdf3DAnnotationR.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DAnnotation_CreatePdf3DAnnotationR,ptrRec)
        super(Pdf3DAnnotation, self).__init__(intPtr)

    @dispatch
    def __init__(self, rectangle: RectangleF, fileName: str):
        """
        Initializes a new instance of the Pdf3DAnnotation class with the specified rectangle and file name.
        
        Args:
            rectangle (RectangleF): The rectangle of the annotation.
            fileName (str): The file name.
        """
        ptrRec: c_void_p = rectangle.Ptr

        GetDllLibPdf().Pdf3DAnnotation_CreatePdf3DAnnotationRF.argtypes = [c_void_p, c_wchar_p]
        GetDllLibPdf().Pdf3DAnnotation_CreatePdf3DAnnotationRF.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DAnnotation_CreatePdf3DAnnotationRF,ptrRec, fileName)
        super(Pdf3DAnnotation, self).__init__(intPtr)

    @property
    def Views(self) -> 'Pdf3DViewCollection':
        """
        Gets the list of available views for the current 3D artwork.
        
        Returns:
            Pdf3DViewCollection: The list of available views.
        """
        GetDllLibPdf().Pdf3DAnnotation_get_Views.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DAnnotation_get_Views.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DAnnotation_get_Views,self.Ptr)
        ret = None if intPtr == None else Pdf3DViewCollection(intPtr)
        return ret

    @property
    def DefaultView(self) -> int:
        """
        Gets or sets the default view.
        
        Returns:
            int: The default view.
        """
        GetDllLibPdf().Pdf3DAnnotation_get_DefaultView.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DAnnotation_get_DefaultView.restype = c_int
        ret = CallCFunction(GetDllLibPdf().Pdf3DAnnotation_get_DefaultView,self.Ptr)
        return ret

    @DefaultView.setter
    def DefaultView(self, value: int):
        """
        Sets the default view.
        
        Args:
            value (int): The default view.
        """
        GetDllLibPdf().Pdf3DAnnotation_set_DefaultView.argtypes = [c_void_p, c_int]
        CallCFunction(GetDllLibPdf().Pdf3DAnnotation_set_DefaultView,self.Ptr, value)

    @property
    def OnInstantiate(self) -> str:
        """
        Gets or sets the code to execute when the 3D artwork is instantiated.
        
        Returns:
            str: The Javascript code to be executed when the 3D artwork is instantiated.
        """
        GetDllLibPdf().Pdf3DAnnotation_get_OnInstantiate.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DAnnotation_get_OnInstantiate.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().Pdf3DAnnotation_get_OnInstantiate,self.Ptr))
        return ret

    @OnInstantiate.setter
    def OnInstantiate(self, value: str):
        """
        Sets the code to execute when the 3D artwork is instantiated.
        
        Args:
            value (str): The Javascript code to be executed when the 3D artwork is instantiated.
        """
        GetDllLibPdf().Pdf3DAnnotation_set_OnInstantiate.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().Pdf3DAnnotation_set_OnInstantiate,self.Ptr, value)

    @property
    def Activation(self) -> 'Pdf3DActivation':
        """
        Gets or sets the activation options for the annotation.
        
        Returns:
            Pdf3DActivation: The activation options for the annotation.
        """
        GetDllLibPdf().Pdf3DAnnotation_get_Activation.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DAnnotation_get_Activation.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DAnnotation_get_Activation,self.Ptr)
        ret = None if intPtr == None else Pdf3DActivation(intPtr)
        return ret

    @Activation.setter
    def Activation(self, value: 'Pdf3DActivation'):
        """
        Sets the activation options for the annotation.
        
        Args:
            value (Pdf3DActivation): The activation options for the annotation.
        """
        GetDllLibPdf().Pdf3DAnnotation_set_Activation.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().Pdf3DAnnotation_set_Activation,self.Ptr, value.Ptr)

    @property
    def _3DData(self) -> Stream:
        """
        Gets a 3D video file from Pdf3DAnnotation.
        
        Returns:
            Stream: The 3D video file.
        """
        GetDllLibPdf().Pdf3DAnnotation_get__3DData.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DAnnotation_get__3DData.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DAnnotation_get__3DData,self.Ptr)
        ret = None if intPtr == None else Stream(intPtr)
        return ret

    @property
    def FileName(self) -> str:
        """
        Gets or sets the filename with full path.
        
        Returns:
            str: The filename with full path.
        """
        GetDllLibPdf().Pdf3DAnnotation_get_FileName.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DAnnotation_get_FileName.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().Pdf3DAnnotation_get_FileName,self.Ptr))
        return ret

    @FileName.setter
    def FileName(self, value: str):
        """
        Sets the filename with full path.
        
        Args:
            value (str): The filename with full path.
        """
        GetDllLibPdf().Pdf3DAnnotation_set_FileName.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().Pdf3DAnnotation_set_FileName,self.Ptr, value)
