from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class Pdf3DView(SpireObject):
    """
    Represents a attributes to be applied to the virtual camera associated with a 3D annotation.
    """
    @dispatch
    def __init__(self):
        GetDllLibPdf().Pdf3DView_CreatePdf3DView.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DView_CreatePdf3DView)
        super(Pdf3DView, self).__init__(intPtr)

    @property
    def Background(self) -> 'Pdf3DBackground':
        """
        Gets or sets the background for this view.
        """
        GetDllLibPdf().Pdf3DView_get_Background.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_Background.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DView_get_Background,self.Ptr)
        ret = None if intPtr == None else Pdf3DBackground(intPtr)
        return ret

    @Background.setter
    def Background(self, value: 'Pdf3DBackground'):
        GetDllLibPdf().Pdf3DView_set_Background.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_Background,self.Ptr, value.Ptr)

    @property
    def CameraToWorldMatrix(self) -> List[float]:
        """
        Gets or sets the 3D transformation matrix.
        """
        GetDllLibPdf().Pdf3DView_get_CameraToWorldMatrix.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_CameraToWorldMatrix.restype = IntPtrArray
        intPtrArray = CallCFunction(GetDllLibPdf().Pdf3DView_get_CameraToWorldMatrix,self.Ptr)
        ret = GetVectorFromArray(intPtrArray, c_float)
        return ret

    @CameraToWorldMatrix.setter
    def CameraToWorldMatrix(self, value: List[float]):
        """
        Sets the 3D transformation matrix.
        """
        vCount = len(value)
        ArrayType = c_float * vCount
        vArray = ArrayType()
        for i in range(0, vCount):
            vArray[i] = value[i]
        GetDllLibPdf().Pdf3DView_set_CameraToWorldMatrix.argtypes = [c_void_p, ArrayType, c_int]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_CameraToWorldMatrix,self.Ptr, vArray, vCount)

    @property
    def CenterOfOrbit(self) -> float:
        """
        Gets or sets the center of orbit for 3D artwork.
        """
        GetDllLibPdf().Pdf3DView_get_CenterOfOrbit.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_CenterOfOrbit.restype = c_float
        ret = CallCFunction(GetDllLibPdf().Pdf3DView_get_CenterOfOrbit,self.Ptr)
        return ret

    @CenterOfOrbit.setter
    def CenterOfOrbit(self, value: float):
        GetDllLibPdf().Pdf3DView_set_CenterOfOrbit.argtypes = [c_void_p, c_float]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_CenterOfOrbit,self.Ptr, value)

    @property
    def CrossSections(self) -> 'Pdf3DCrossSectionCollection':
        """
        Gets the list of cross sections for this view.
        """
        GetDllLibPdf().Pdf3DView_get_CrossSections.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_CrossSections.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DView_get_CrossSections,self.Ptr)
        ret = None if intPtr == None else Pdf3DCrossSectionCollection(intPtr)
        return ret

    @property
    def ExternalName(self) -> str:
        """
        Gets or sets the view's external name.
        """
        GetDllLibPdf().Pdf3DView_get_ExternalName.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_ExternalName.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().Pdf3DView_get_ExternalName,self.Ptr))
        return ret

    @ExternalName.setter
    def ExternalName(self, value: str):
        GetDllLibPdf().Pdf3DView_set_ExternalName.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_ExternalName,self.Ptr, value)

    @property
    def InternalName(self) -> str:
        """
        Gets or sets the view's internal name.
        """
        GetDllLibPdf().Pdf3DView_get_InternalName.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_InternalName.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().Pdf3DView_get_InternalName,self.Ptr))
        return ret

    @InternalName.setter
    def InternalName(self, value: str):
        GetDllLibPdf().Pdf3DView_set_InternalName.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_InternalName,self.Ptr, value)

    @property
    def LightingScheme(self) -> 'Pdf3DLighting':
        """
        Gets or sets the Creates a new page and adds it as the last page of the document scheme for this view.
        """
        GetDllLibPdf().Pdf3DView_get_LightingScheme.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_LightingScheme.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DView_get_LightingScheme,self.Ptr)
        ret = None if intPtr == None else Pdf3DLighting(intPtr)
        return ret

    @LightingScheme.setter
    def LightingScheme(self, value: 'Pdf3DLighting'):
        GetDllLibPdf().Pdf3DView_set_LightingScheme.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_LightingScheme,self.Ptr, value.Ptr)

    @property
    def Nodes(self) -> 'Pdf3DNodeCollection':
        """
        Gets the list of 3D nodes for this view.
        """
        GetDllLibPdf().Pdf3DView_get_Nodes.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_Nodes.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DView_get_Nodes,self.Ptr)
        ret = None if intPtr == None else Pdf3DNodeCollection(intPtr)
        return ret

    @property
    def Projection(self) -> 'Pdf3DProjection':
        """
        Gets or sets the projection for this view.
        """
        GetDllLibPdf().Pdf3DView_get_Projection.argtypes = [c_void_p]
        GetDllLibPdf().Pdf3DView_get_Projection.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DView_get_Projection,self.Ptr)
        ret = None if intPtr == None else Pdf3DProjection(intPtr)
        return ret

    @Projection.setter
    def Projection(self, value:'Pdf3DProjection'):
        GetDllLibPdf().Pdf3DView_set_Projection.argtypes=[c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_Projection,self.Ptr, value.Ptr)

    @property

    def RenderMode(self)->'Pdf3DRendermode':
        """

        Gets or sets the rendering mode for this view. 

        """
        GetDllLibPdf().Pdf3DView_get_RenderMode.argtypes=[c_void_p]
        GetDllLibPdf().Pdf3DView_get_RenderMode.restype=c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Pdf3DView_get_RenderMode,self.Ptr)
        ret = None if intPtr==None else Pdf3DRendermode(intPtr)
        return ret


    @RenderMode.setter
    def RenderMode(self, value:'Pdf3DRendermode'):
        GetDllLibPdf().Pdf3DView_set_RenderMode.argtypes=[c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_RenderMode,self.Ptr, value.Ptr)

    @property
    def ResetNodesState(self)->bool:
        """

        Gets or sets a value indicating whether nodes specified in the Nodes collection are returned to their original states (as specified in the 3D artwork) before applying transformation matrices and opacity settings specified in the node dictionaries. 

        """
        GetDllLibPdf().Pdf3DView_get_ResetNodesState.argtypes=[c_void_p]
        GetDllLibPdf().Pdf3DView_get_ResetNodesState.restype=c_bool
        ret = CallCFunction(GetDllLibPdf().Pdf3DView_get_ResetNodesState,self.Ptr)
        return ret

    @ResetNodesState.setter
    def ResetNodesState(self, value:bool):
        GetDllLibPdf().Pdf3DView_set_ResetNodesState.argtypes=[c_void_p, c_bool]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_ResetNodesState,self.Ptr, value)

    @property

    def ViewNodeName(self)->str:
        """

        Gets or sets the name of the view node.         

        The view node in the content stream defines all the properties for viewing the 3D artwork. If both ViewNodeName and CameraToWorldMatrix are specified, then ViewNodeName takes precedence.</remarks>
        """
        GetDllLibPdf().Pdf3DView_get_ViewNodeName.argtypes=[c_void_p]
        GetDllLibPdf().Pdf3DView_get_ViewNodeName.restype=c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().Pdf3DView_get_ViewNodeName,self.Ptr))
        return ret


    @ViewNodeName.setter
    def ViewNodeName(self, value:str):
        GetDllLibPdf().Pdf3DView_set_ViewNodeName.argtypes=[c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().Pdf3DView_set_ViewNodeName,self.Ptr, value)