from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class PdfAnnotationCollection(PdfCollection):
    """
    Represents collection of objects.
    """

    def get_Item(self, index: int) -> 'PdfAnnotation':
        """
        Gets the object at the specified position.

        Args:
            index: The index value of the annotation in the collection.

        Returns:
            Annotation object at the specified position.
        """
        GetDllLibPdf().PdfAnnotationCollection_get_Item.argtypes = [c_void_p, c_int]
        GetDllLibPdf().PdfAnnotationCollection_get_Item.restype = IntPtrWithTypeName
        intPtr = CallCFunction(GetDllLibPdf().PdfAnnotationCollection_get_Item,self.Ptr, index)
        ret = None if intPtr == None else self._create(intPtr)
        return ret

    def _create(self, intPtrWithTypeName: IntPtrWithTypeName) -> PdfAnnotation:
        """
        Creates a PdfAnnotation object based on the given IntPtrWithTypeName.

        Args:
            intPtrWithTypeName: The IntPtrWithTypeName object.

        Returns:
            The created PdfAnnotation object.
        """
        ret = None
        if intPtrWithTypeName == None:
            return ret
        intPtr = intPtrWithTypeName.intPtr[0] + (intPtrWithTypeName.intPtr[1] << 32)
        strName = PtrToStr(intPtrWithTypeName.typeName)
        if strName == 'Spire.Pdf.Annotations.PdfTextMarkupAnnotationWidget':
            ret = PdfTextMarkupAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfFreeTextAnnotationWidget':
            ret = PdfFreeTextAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.Pdf3DAnnotation':
            ret = Pdf3DAnnotation(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfPopupAnnotationWidget':
            ret = PdfPopupAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfTextWebLinkAnnotationWidget':
            ret = PdfTextWebLinkAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfRubberStampAnnotationWidget':
            ret = PdfRubberStampAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfTextAnnotationWidget':
            ret = PdfTextAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfAttachmentAnnotationWidget':
            ret = PdfAttachmentAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfLineAnnotationWidget':
            ret = PdfLineAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfDocumentLinkAnnotationWidget':
            ret = PdfDocumentLinkAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfPolygonAnnotationWidget':
            ret = PdfPolygonAnnotationWidget(intPtr)
        elif strName == 'Spire.Pdf.Annotations.PdfUriAnnotationWidget':
            ret = PdfUriAnnotationWidget(intPtr)
        else:
            ret = PdfAnnotation(intPtr)
        return ret

    def Add(self, annotation: 'PdfAnnotation') -> int:
        """
        Adds a new annotation to collection.

        Args:
            annotation: The new annotation to be added to collection.

        Returns:
            Position of the annotation in collection.
        """
        intPtrannotation: c_void_p = annotation.Ptr

        GetDllLibPdf().PdfAnnotationCollection_Add.argtypes = [c_void_p, c_void_p]
        GetDllLibPdf().PdfAnnotationCollection_Add.restype = c_int
        ret = CallCFunction(GetDllLibPdf().PdfAnnotationCollection_Add,self.Ptr, intPtrannotation)
        return ret

    def Clear(self):
        """
        Clears the collection.
        """
        GetDllLibPdf().PdfAnnotationCollection_Clear.argtypes = [c_void_p]
        CallCFunction(GetDllLibPdf().PdfAnnotationCollection_Clear,self.Ptr)

    def Contains(self, annotation: 'PdfAnnotation') -> bool:
        """
        Searches the collection for the specified annotation.

        Args:
            annotation: The annotation to search for.

        Returns:
            True, if annotation is contained in collection. Otherwise - false.
        """
        intPtrannotation: c_void_p = annotation.Ptr

        GetDllLibPdf().PdfAnnotationCollection_Contains.argtypes = [c_void_p, c_void_p]
        GetDllLibPdf().PdfAnnotationCollection_Contains.restype = c_bool
        ret = CallCFunction(GetDllLibPdf().PdfAnnotationCollection_Contains,self.Ptr, intPtrannotation)
        return ret

    def IndexOf(self, annotation: 'PdfAnnotation') -> int:
        """
        Searches the collection for the specified annotation.

        Args:
            annotation: The Annotation to search.

        Returns:
            Index of the element in the collection, if exists, or -1 if the element does not exist in the collection.
        """
        intPtrannotation: c_void_p = annotation.Ptr

        GetDllLibPdf().PdfAnnotationCollection_IndexOf.argtypes = [c_void_p, c_void_p]
        GetDllLibPdf().PdfAnnotationCollection_IndexOf.restype = c_int
        ret = CallCFunction(GetDllLibPdf().PdfAnnotationCollection_IndexOf,self.Ptr, intPtrannotation)
        return ret

    def Insert(self, index: int, annotation: 'PdfAnnotation'):
        """
        Inserts annotation to the collection at the specified index.

        Args:
            index: Index where to insert the element.
            annotation: The annotation to insert in the collection.
        """
        intPtrannotation: c_void_p = annotation.Ptr

        GetDllLibPdf().PdfAnnotationCollection_Insert.argtypes = [c_void_p, c_int, c_void_p]
        CallCFunction(GetDllLibPdf().PdfAnnotationCollection_Insert,self.Ptr, index, intPtrannotation)

    def RemoveAt(self, index: int):
        """
        Removes the element at the specified field.

        Args:
            index: The index of the element to remove.
        """
        GetDllLibPdf().PdfAnnotationCollection_RemoveAt.argtypes = [c_void_p, c_int]
        CallCFunction(GetDllLibPdf().PdfAnnotationCollection_RemoveAt,self.Ptr, index)

    def Remove(self, annot: 'PdfAnnotation'):
        """
        Removes the element from the collection.

        Args:
            field: The element to remove.
        """
        intPtrannot: c_void_p = annot.Ptr

        GetDllLibPdf().PdfAnnotationCollection_Remove.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfAnnotationCollection_Remove,self.Ptr, intPtrannot)