from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class PdfCalRGBColorSpace(PdfColorSpaces):
    """
    Represents a CalRGB colorspace.
    """

    @property
    def BlackPoint(self) -> List[float]:
        """
        Gets or sets the black point.

        Returns:
            An array of three numbers [XB YB ZB] specifying the tristimulus value, in the CIE 1931 XYZ space, of the diffuse black point.
        """
        GetDllLibPdf().PdfCalRGBColorSpace_get_BlackPoint.argtypes = [c_void_p]
        GetDllLibPdf().PdfCalRGBColorSpace_get_BlackPoint.restype = IntPtrArray
        intPtrArray = CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_get_BlackPoint,self.Ptr)
        ret = GetVectorFromArray(intPtrArray, c_double)
        return ret

    @BlackPoint.setter
    def BlackPoint(self, value: List[float]):
        """
        Sets the black point.

        Args:
            value: An array of three numbers [XB YB ZB] specifying the tristimulus value, in the CIE 1931 XYZ space, of the diffuse black point.
        """
        vCount = len(value)
        ArrayType = c_double * vCount
        vArray = ArrayType()
        for i in range(0, vCount):
            vArray[i] = value[i]
        GetDllLibPdf().PdfCalRGBColorSpace_set_BlackPoint.argtypes = [c_void_p, ArrayType, c_int]
        CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_set_BlackPoint,self.Ptr, vArray, vCount)

    @property
    def Gamma(self) -> List[float]:
        """
        Gets or sets the gamma.

        Returns:
            An array of three numbers [GR GG GB] specifying the gamma for the red, green, and blue components of the color space.
        """
        GetDllLibPdf().PdfCalRGBColorSpace_get_Gamma.argtypes = [c_void_p]
        GetDllLibPdf().PdfCalRGBColorSpace_get_Gamma.restype = IntPtrArray
        intPtrArray = CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_get_Gamma,self.Ptr)
        ret = GetVectorFromArray(intPtrArray, c_double)
        return ret

    @Gamma.setter
    def Gamma(self, value: List[float]):
        """
        Sets the gamma.

        Args:
            value: An array of three numbers [GR GG GB] specifying the gamma for the red, green, and blue components of the color space.
        """
        vCount = len(value)
        ArrayType = c_double * vCount
        vArray = ArrayType()
        for i in range(0, vCount):
            vArray[i] = value[i]
        GetDllLibPdf().PdfCalRGBColorSpace_set_Gamma.argtypes = [c_void_p, ArrayType, c_int]
        CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_set_Gamma,self.Ptr, vArray, vCount)

    @property
    def Matrix(self) -> List[float]:
        """
        Gets or sets the colorspace transformation matrix.

        Returns:
            An array of nine numbers [XA YA ZA XB YB ZB XC YC ZC] specifying the linear interpretation of the decoded A, B, and C components of the color space with respect to the final XYZ representation.
        """
        GetDllLibPdf().PdfCalRGBColorSpace_get_Matrix.argtypes = [c_void_p]
        GetDllLibPdf().PdfCalRGBColorSpace_get_Matrix.restype = IntPtrArray
        intPtrArray = CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_get_Matrix,self.Ptr)
        ret = GetVectorFromArray(intPtrArray, c_double)
        return ret

    @Matrix.setter
    def Matrix(self, value: List[float]):
        """
        Sets the colorspace transformation matrix.

        Args:
            value: An array of nine numbers [XA YA ZA XB YB ZB XC YC ZC] specifying the linear interpretation of the decoded A, B, and C components of the color space with respect to the final XYZ representation.
        """
        vCount = len(value)
        ArrayType = c_double * vCount
        vArray = ArrayType()
        for i in range(0, vCount):
            vArray[i] = value[i]
        GetDllLibPdf().PdfCalRGBColorSpace_set_Matrix.argtypes = [c_void_p, ArrayType, c_int]
        CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_set_Matrix,self.Ptr, vArray, vCount)

    @property
    def WhitePoint(self) -> List[float]:
        """
        Gets or sets the white point.

        Returns:
            An array of three numbers [XW YW ZW] specifying the tristimulus value, in the CIE 1931 XYZ space, of the diffuse white point.
        """
        GetDllLibPdf().PdfCalRGBColorSpace_get_WhitePoint.argtypes = [c_void_p]
        GetDllLibPdf().PdfCalRGBColorSpace_get_WhitePoint.restype = IntPtrArray
        intPtrArray = CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_get_WhitePoint,self.Ptr)
        ret = GetVectorFromArray(intPtrArray, c_double)
        return ret

    @WhitePoint.setter
    def WhitePoint(self, value: List[float]):
        """
        Sets the white point.

        Args:
            value: An array of three numbers [XW YW ZW] specifying the tristimulus value, in the CIE 1931 XYZ space, of the diffuse white point.
        """
        vCount = len(value)
        ArrayType = c_double * vCount
        vArray = ArrayType()
        for i in range(0, vCount):
            vArray[i] = value[i]
        GetDllLibPdf().PdfCalRGBColorSpace_set_WhitePoint.argtypes = [c_void_p, ArrayType, c_int]
        CallCFunction(GetDllLibPdf().PdfCalRGBColorSpace_set_WhitePoint,self.Ptr, vArray, vCount)