from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class PdfMargins(SpireObject):
    """
    A class representing page margins.
    """

    @dispatch
    def __init__(self):
        """
        Initializes a new instance of the PdfMargins class.
        """
        GetDllLibPdf().PdfMargins_CreatePdfMargins.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfMargins_CreatePdfMargins)
        super(PdfMargins, self).__init__(intPtr)

    @dispatch
    def __init__(self, margin: float):
        """
        Initializes a new instance of the PdfMargins class with a specified margin size.
        
        :param margin: The margin size.
        """
        GetDllLibPdf().PdfMargins_CreatePdfMarginsM.argtypes = [c_float]
        GetDllLibPdf().PdfMargins_CreatePdfMarginsM.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfMargins_CreatePdfMarginsM,margin)
        super(PdfMargins, self).__init__(intPtr)

    @dispatch
    def __init__(self, leftRight: float, topBottom: float):
        """
        Initializes a new instance of the PdfMargins class with specified left/right and top/bottom margin sizes.
        
        :param leftRight: The left/right margin size.
        :param topBottom: The top/bottom margin size.
        """
        GetDllLibPdf().PdfMargins_CreatePdfMarginsLT.argtypes = [c_float, c_float]
        GetDllLibPdf().PdfMargins_CreatePdfMarginsLT.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfMargins_CreatePdfMarginsLT,leftRight, topBottom)
        super(PdfMargins, self).__init__(intPtr)

    @dispatch
    def __init__(self, left: float, top: float, right: float, bottom: float):
        """
        Initializes a new instance of the PdfMargins class with specified left, top, right, and bottom margin sizes.
        
        :param left: The left margin size.
        :param top: The top margin size.
        :param right: The right margin size.
        :param bottom: The bottom margin size.
        """
        GetDllLibPdf().PdfMargins_CreatePdfMarginsLTRB.argtypes = [c_float, c_float, c_float, c_float]
        GetDllLibPdf().PdfMargins_CreatePdfMarginsLTRB.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfMargins_CreatePdfMarginsLTRB,left, top, right, bottom)
        super(PdfMargins, self).__init__(intPtr)

    @property
    def Left(self) -> float:
        """
        Gets or sets the left margin size.
        """
        GetDllLibPdf().PdfMargins_get_Left.argtypes = [c_void_p]
        GetDllLibPdf().PdfMargins_get_Left.restype = c_float
        ret = CallCFunction(GetDllLibPdf().PdfMargins_get_Left,self.Ptr)
        return ret

    @Left.setter
    def Left(self, value: float):
        """
        Sets the left margin size.
        
        :param value: The left margin size.
        """
        GetDllLibPdf().PdfMargins_set_Left.argtypes = [c_void_p, c_float]
        CallCFunction(GetDllLibPdf().PdfMargins_set_Left,self.Ptr, value)

    @property
    def Top(self) -> float:
        """
        Gets or sets the top margin size.
        """
        GetDllLibPdf().PdfMargins_get_Top.argtypes = [c_void_p]
        GetDllLibPdf().PdfMargins_get_Top.restype = c_float
        ret = CallCFunction(GetDllLibPdf().PdfMargins_get_Top,self.Ptr)
        return ret

    @Top.setter
    def Top(self, value: float):
        """
        Sets the top margin size.
        
        :param value: The top margin size.
        """
        GetDllLibPdf().PdfMargins_set_Top.argtypes = [c_void_p, c_float]
        CallCFunction(GetDllLibPdf().PdfMargins_set_Top,self.Ptr, value)

    @property
    def Right(self) -> float:
        """
        Gets or sets the right margin size.
        """
        GetDllLibPdf().PdfMargins_get_Right.argtypes = [c_void_p]
        GetDllLibPdf().PdfMargins_get_Right.restype = c_float
        ret = CallCFunction(GetDllLibPdf().PdfMargins_get_Right,self.Ptr)
        return ret

    @Right.setter
    def Right(self, value: float):
        """
        Sets the right margin size.
        
        :param value: The right margin size.
        """
        GetDllLibPdf().PdfMargins_set_Right.argtypes = [c_void_p, c_float]
        CallCFunction(GetDllLibPdf().PdfMargins_set_Right,self.Ptr, value)

    @property
    def Bottom(self) -> float:
        """
        Gets or sets the bottom margin size.
        """
        GetDllLibPdf().PdfMargins_get_Bottom.argtypes = [c_void_p]
        GetDllLibPdf().PdfMargins_get_Bottom.restype = c_float
        ret = CallCFunction(GetDllLibPdf().PdfMargins_get_Bottom,self.Ptr)
        return ret

    @Bottom.setter
    def Bottom(self, value: float):
        """
        Sets the bottom margin size.
        
        :param value: The bottom margin size.
        """
        GetDllLibPdf().PdfMargins_set_Bottom.argtypes = [c_void_p, c_float]
        CallCFunction(GetDllLibPdf().PdfMargins_set_Bottom,self.Ptr, value)

    @property
    def All(self) -> float:
        """
        Gets or sets the margin size for all sides.
        """
        return 0

    @All.setter
    def All(self, value: float):
        """
        Sets the margin size for all sides.
        
        :param value: The margin size for all sides.
        """
        GetDllLibPdf().PdfMargins_set_All.argtypes = [c_void_p, c_float]
        CallCFunction(GetDllLibPdf().PdfMargins_set_All,self.Ptr, value)

    def Clone(self) -> 'SpireObject':
        """
        Clones the object.
        
        :returns: The cloned object.
        """
        GetDllLibPdf().PdfMargins_Clone.argtypes = [c_void_p]
        GetDllLibPdf().PdfMargins_Clone.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfMargins_Clone,self.Ptr)
        ret = None if intPtr == None else SpireObject(intPtr)
        return ret